/*
 * StateNonterminalMap.h
 *
 *  Created on: 15. 3. 2014
 *      Author: tomas
 */

#ifndef STATENONTERMINALMAP_H_
#define STATENONTERMINALMAP_H_

#include <map>

#include <automaton/FSM/FSM.h>
#include <grammar/Regular/RegularGrammar.h>
#include <AlibException.h>

#include "../include/macros.h"

namespace conversions
{

/**
 * This class maps grammar nonterminals to automata states in order to avoid
 * symbol collision.
 */
class StateNonterminalMap
{
public:
    /**
     * Initiates mapping function, also creates nonterminals states in target grammar.
     *
     * @param fsm Source FSM.
     * @param grammar Target grammar.
     */
    StateNonterminalMap( const automaton::FSM & fsm, grammar::RegularGrammar & grammar );
    const alphabet::Symbol & getNonTerminal( const automaton::State & state );

private:
    void initMap( void );

    /**
     * Maps automaton's state to grammar's Symbol
     */
    std::map<const automaton::State, const alphabet::Symbol> m_map;

    /**
     * Source automata.
     */
    const automaton::FSM m_fsm;

    /**
     * Target grammar.
     */
    grammar::RegularGrammar & m_grammar;

};

} /* namespace conversions */

#endif /* STATENONTERMINALMAP_H_ */
