/*
 * Brzozowski.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKI_H_
#define BRZOZOWSKI_H_

#include <map>
#include <set>
#include <deque>

#include <automaton/State.h>
#include <AlibException.h>

#include "AbstractREtoFAConverter.h"

#include "RegExpDerivation.h"

#include "RegExpOptimize.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Brzozowski algorithm (derivations of regular expressions).
 * Source: Melichar 2.110
 */
class Brzozowski : public AbstractREtoFAConverter
{
public:
    /**
     * @param re Source regular expression.
     */
    Brzozowski( const regexp::RegExp & re );

    ~Brzozowski( void );

    /**
     * Performs conversion.
     * @return FSM equivalent to original regular expression.
     */
    automaton::FSM convert( void );

private:
    /**
     * Maps every regular expression to unique state of automata.
     */
    class StateBuilder
    {
    public:
        /**
         * @param Q set of regular expressions.
         */
        StateBuilder( const std::set<regexp::RegExp> & Q );

        /**
         * Get automata state representing given regular expression.
         *
         * @param re Regular expression
         * @return automata state representing given regular expression.
         */
        const automaton::State & getState( const regexp::RegExp & re ) const;

    private:
        /**
         * Creates original state name.
         */
        std::string createNewName( void );

        /**
         * Map
         */
        std::map<const regexp::RegExp, automaton::State> m_states;

        unsigned int m_stateId;
    };

    /**
     * Stores original regular epxression (m_from), and result of derivation (m_to) over m_symbol.
     */
    struct Transition
    {
        const regexp::RegExp m_from, m_to;
        const alphabet::Symbol m_symbol;

        /**
         * @param from original regexp
         * @param to resulting regexp
         * @param symb
         */
        Transition( const regexp::RegExp & from, const alphabet::Symbol & symb, const regexp::RegExp & to );

        bool operator<( const Transition & other ) const;
    };

    /**
     * Set of transitions.
     * @see Brzozowski::Transition
     */
    std::set<Transition> m_transitions;
};

} /* namespace conversions */

#endif /* BRZOZOWSKI_H_ */
