/*
 * Glushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#include "Glushkov.h"

using namespace alib;
using namespace alphabet;
using namespace automaton;
using namespace regexp;
using namespace std;

namespace conversions
{

Glushkov::Glushkov( const RegExp & re ) : AbstractREtoFAConverter( re )
{

}

Glushkov::~Glushkov( void )
{

}

FSM Glushkov::convert( void )
{
    // step 1
    initNumberSymbols( );

    for( const auto & symbol : m_re.getAlphabet( ) )
        m_fsm.addInputSymbol( symbol.getSymbol( ) );

    // steps 2, 3, 4
    constructBeginSymbolSet( );
    constructNeighbourSymbolSet( );
    constructEndSymbolSet( );
        // \e in q0 check is in step 7

    // step 5
    State q0( "q0" );
    m_fsm.addState( q0 );
    m_fsm.addInitialState( q0 );
    for( const auto & kv : m_numberedSymbols )
        m_fsm.addState( kv.second.m_state );

    // step 6
    for( const auto & ns : m_beginSymbolSet )
        m_fsm.addTransition( q0, ns.m_alphabetSymbol, ns.m_state );

    for( const auto & ns : m_neighbourSymbolSet )
    {
        const NumberedSymbol & first  = m_numberedSymbols.find( ns.m_first )->second;
        const NumberedSymbol & second = m_numberedSymbols.find( ns.m_second )->second;

        m_fsm.addTransition( first.m_state, second.m_alphabetSymbol, second.m_state );
    }

    // step 7
    for( const auto & ns : m_endSymbolSet )
        m_fsm.addFinalState( State ( ns.m_state ) );

    if( m_re.containsEmptyString( ) )
        m_fsm.addFinalState( q0 );

    return m_fsm;
}

void Glushkov::initNumberSymbols( void )
{
    int id = 1;

    for( const auto & symb : GlushkovTraversal::getSymbolsListInOrder( m_re ) )
        m_numberedSymbols.insert( pair<const RegExpSymbol*, NumberedSymbol>( symb, NumberedSymbol( symb, id ++ ) ) );
}

void Glushkov::constructBeginSymbolSet( void )
{
    for( const auto & s : GlushkovTraversal::getLeftmostSymbolsInTree( m_re ) )
        m_beginSymbolSet.insert( m_numberedSymbols.find( s )->second );
}

void Glushkov::constructEndSymbolSet( void )
{
    for( const auto & s : GlushkovTraversal::getRightmostSymbolsInTree( m_re ) )
        m_endSymbolSet.insert( m_numberedSymbols.find( s )->second );
}

void Glushkov::constructNeighbourSymbolSet( void )
{
    for( const auto & n : GlushkovTraversal::getNeighbours( m_re ) )
        m_neighbourSymbolSet.insert( n );
}

// ----------------------------------------------------------------------------

bool Glushkov::NumberedSymbol::operator<( const NumberedSymbol & x ) const
{
    return m_i < x.m_i;
}

Glushkov::NumberedSymbol::NumberedSymbol( const RegExpSymbol * symbol, int i ) :
        m_i( i ),
        m_alphabetSymbol( symbol->getSymbol( ) ),
        //m_state( State( "[" + m_alphabetSymbol.getSymbol( ) + "], id " + to_string( m_i ) ) )
        m_state( State( to_string( m_i ) ) )
{

}

} /* namespace conversions */
