#include "RegExpPrinter.h"

RegExpPrinter::RegExpPrinter(std::ostream& m_Out) : m_Out(m_Out) {
  
}

void RegExpPrinter::printElement(regexp::RegExpElement* element) {
  regexp::Alternation* alternation = dynamic_cast<regexp::Alternation*>(element);
  if (alternation) {
	  printAlternation(alternation);
	  return;
  }

  regexp::Concatenation* concatenation = dynamic_cast<regexp::Concatenation*>(element);
  if(concatenation) {
	  printConcatenation(concatenation);
	  return;
  }

  regexp::Iteration* iteration = dynamic_cast<regexp::Iteration*>(element);
  if (iteration) {
	  printIteration(iteration);
	  return;
  }

  regexp::RegExpEpsilon* epsilon = dynamic_cast<regexp::RegExpEpsilon*>(element);
  if (epsilon) {
	  printEpsilon(epsilon);
	  return;
  }

  regexp::RegExpEmpty* empty = dynamic_cast<regexp::RegExpEmpty*>(element);
  if (empty) {
	  printEmpty(empty);
	  return;
  }

  regexp::RegExpSymbol* symbol = dynamic_cast<regexp::RegExpSymbol*>(element);
  if (symbol) {
	  printSymbol(symbol);
	  return;
  }
}

void RegExpPrinter::printAlternation(regexp::Alternation* alternation) {
  m_Out << "(";
  bool first = true;
  for (auto element : alternation->getElements()) {
    if(first) {
      first = false;
    } else {
      m_Out << "+";
    }
    printElement(element);
  }
  m_Out << ")";
}

void RegExpPrinter::printConcatenation(regexp::Concatenation* concatenation) {
  for (auto element : concatenation->getElements()) {
    printElement(element);
  }
}

void RegExpPrinter::printIteration(regexp::Iteration* iteration) {
  m_Out << "(";
  printElement(iteration->getElement());
  m_Out << ")*";
}

void RegExpPrinter::printSymbol(regexp::RegExpSymbol* symbol) {
  m_Out << symbol->getSymbol();
}

void RegExpPrinter::printEpsilon(regexp::RegExpEpsilon* symbol) {
  m_Out << "\\e";
}

void RegExpPrinter::printEmpty(regexp::RegExpEmpty* symbol) {
  m_Out << "\\0";
}

void RegExpPrinter::print(regexp::RegExp& regexp) {
  printElement(regexp.getRegExp());
  m_Out << std::endl;
}