#include "FsmDeterminizer.h"

namespace determinization {
namespace fsm {


void FsmDeterminizer::initDeterminization()
{
    this->QPrimed.clear();
    Utils::copyInputAlphabet(*this->nfsm, *this->dfsm);
}


const State& FsmDeterminizer::getOrCreateState(const set<State>& originalStates)
{
    return FsmUtils::getOrCreateState(originalStates, this->QPrimed, *this->dfsm);
}


set<State> FsmDeterminizer::findToStates(const set<State>& fromStates, const Symbol& input)
{
    set<State> toStates;
    for (const auto& state : fromStates) {
        for (const auto& transition : this->nfsm->getTransitions()) {
            if (transition.getFrom() == state && transition.getInput() == input) {
                toStates.insert(transition.getTo());
            }
        }
    }
    return toStates;
}


FsmDeterminizer::FsmDeterminizer(FSM* nfsm)
{
    this->nfsm = nfsm;
}


Automaton* FsmDeterminizer::determinize()
{
    this->dfsm = new FSM();
    this->initDeterminization();
    
    // 1
    const set<State>& q0 = this->nfsm->getInitialStates();
    const State& q0Primed = this->getOrCreateState(q0);
    
    while (true) {
        // 2
        bool allStatesMarked = true;
        for (const auto& stateIter : this->QPrimed) {
            allStatesMarked &= stateIter.second.isMarked;
        }
        if (allStatesMarked) {
            break;
        }
        
        // 3
        StateData* qPrimed = NULL;
        for (auto& stateIter : this->QPrimed) {
            StateData& stateData = stateIter.second;
            if (!stateData.isMarked) {
                qPrimed = &stateData;
                break;
            }
        }
        
        // 3a and 3b
        for (const auto& input : this->nfsm->getInputAlphabet()) {
            const set<State>& nfsmToStates = this->findToStates(qPrimed->originalStates, input);
            if (nfsmToStates.size() > 0) {
                const State& toState = this->getOrCreateState(nfsmToStates);
                TransitionFSM transition(qPrimed->state, input, toState);
                this->dfsm->addTransition(transition);
            }
        }

        // 3c
        qPrimed->isMarked = true;
    }
    
    // 4
    this->dfsm->addInitialState(q0Primed);
    
    // 5
    for (const auto& qPrimedIter : this->QPrimed) {
        const StateData& qPrimed = qPrimedIter.second;
        if (Utils::containFinalState(qPrimed.originalStates, *this->nfsm)) {
            this->dfsm->addFinalState(qPrimed.state);
        }
    }
    
    return this->dfsm;
}


}
}
