#ifndef IDPDADETERMINIZER_H_
#define IDPDADETERMINIZER_H_

#include <string>
#include <set>
#include <map>
#include <list>

#include "automaton/State.h"
#include "automaton/PDA/PDA.h"
#include "automaton/PDA/TransitionPDA.h"

#include "../common/Utils.h"
#include "../Determinizer.h"
#include "../fsm/FsmUtils.h"
#include "../fsm/FsmStructs.h"

using namespace std;
using namespace automaton;
using namespace determinization;
using namespace determinization::fsm;

namespace determinization {
namespace idpda {

/**
 * Class for running determinization algorithm on idpda.
 */
class IdpdaDeterminizer : public Determinizer
{
    private:

        /** Nondeterministic input-driven pushdown automaton */
        PDA* nidpda;

        /** Deterministinc input-driven pushdown automaton */
        PDA* didpda;

        /** Map of states of deterministic idpda, where key is the name of state and value is data about this state */
        map<string, StateData> QPrimed;

        /** Map of pop symbols for input symbols of deterministic idpda */
        map<Symbol, list<Symbol>> popSymbols;

        /** Map of push symbols for input symbols of deterministic idpda */
        map<Symbol, list<Symbol>> pushSymbols;
        
        /**
         * Runs some initializiation stuff before determinization algorithm.
         */
        void initDeterminization();

        /**
         * Finds and saves pop and push symbols for all symbols from input alphabet of idpda.
         */
        void findPopAndPushSymbols();

        /**
         * Returns existing state from states map, if there is one with same name, or creates new one and adds it into
         *     states map and deterministic idpda.
         *
         * @param originalStates set of states from nondeterministic idpda which represents state of deterministic idpda
         * @return state of deterministic idpda
         */
        const State& getOrCreateState(const set<State>& originalStates);

        /**
         * Finds states from nondeterministic idpda to which at least one state from given set of states have transition
         *     with given input.
         * 
         * @param fromStates set of states from nondeterministic idpda
         * @param input symbol from input alphabet
         * @return set of states from nondeterministic idpda
         */
        set<State> findToStates(const set<State>& fromStates, const Symbol& input);
        
    public:

        /**
         * @param nidpda nondeterministic input-driven pushdown automaton given for determinization
         */
        IdpdaDeterminizer(PDA* nidpda);

        /**
         * Runs determinization algorithm on nondeterministic idpda given in constructor.
         *
         * @return deterministic input-driven pushdown automaton
         */
        Automaton* determinize();

};

}
}
#endif
