#ifndef RHDPDADETERMINZER_H_
#define RHDPDADETERMINZER_H_

#include <string>
#include <set>
#include <map>
#include <vector>

#include "automaton/PDA/PDA.h"
#include "automaton/Transition.h"
#include "automaton/exception/AutomatonException.h"
#include "alphabet/Symbol.h"

#include "../common/Utils.h"
#include "../Determinizer.h"
#include "RhdpdaUtils.h"
#include "RhdpdaStructs.h"

using namespace std;
using namespace automaton;
using namespace alphabet;

namespace determinization {
namespace rhdpda {

class RhdpdaDeterminizer : public Determinizer
{
    private:
        PDA* rhdpda;
        PDA* rdpda;
        map<string, StateData> states;
        set<TransitionPDA> internalTransitions;
        set<TransitionPDA> pushTransitions;
        set<TransitionPDA> popTransitions;
        set<StateSymbolPair> allStateSymbolPairs;
        vector<SComponent> allSComponents;
        vector<RComponent> allRComponents;

        void divideTransitions();
        void generateTransitionsForAllPossibleStackSymbols(const TransitionPDA& sampleTransition);
        string buildStackSymbolName(const SComponent& s, const RComponent& r, const Symbol& input);
        set<StateSymbolPair> combineStatesWithSymbol(const set<State>& states, const Symbol& symbol);
        const SComponent getSComponentWithPairsIdentity(const set<StateSymbolPair>& pairs);
        const State& getOrCreateState(const SComponent& s, const RComponent& r);
        void initDeterminization();

    public:
        RhdpdaDeterminizer(PDA* rhdpda);
        Automaton* determinize();
};

}
}
#endif
