/*
 * EpsilonClosure.cpp
 *
 *  Created on: 16. 1. 2014
 *      Author: tomas
 */

#include "EpsilonClosure.h"

using namespace alib;
using namespace automaton;
using namespace std;

namespace epsilon
{

EpsilonClosure::EpsilonClosure( const FSM & fsm ) : m_fsm( fsm )
{
    for( const auto & q : fsm.getStates( ) )
        closure( q );
}

void EpsilonClosure::closure( const State & q )
{
    queue<State> queue;
    map<State, bool> visited;

    for( const auto & p : m_fsm.getStates( ) )
        visited[ p ] = false;

    queue.push( q );
    while( ! queue.empty( ) )
    {
        const State & p = queue.front( );
        queue.pop( );
        visited[ p ] = true;
        m_closure[ q ].insert( p );

        for( const auto & transition : m_fsm.getTransitionsFromState( p ) )
            if( transition.getInput( ) == Symbol( "" ) && visited [ transition.getTo( ) ] == false )
                queue.push( transition.getTo( ) );
    }
}

const set<State> & EpsilonClosure::getClosure( const State & state ) const
{
    if( m_closure.find( state ) == m_closure.end( ) )
        throw AlibException( "EpsilonClosure: Trying to get closure of state not in automata." );

    return m_closure.at( state );
}

} /* namespace epsilon */
