/*
 * StringFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#include <iostream>

#include "sax/SaxInterface.h"
#include "StringFactory.h"
#include "string/StringParser.h"

namespace notions {
namespace string {

String StringFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxInterface::parseFile(filename, tokens);
	return parse(tokens);
}

String StringFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxInterface::parseMemory(str, tokens);
	return parse(tokens);
}

String StringFactory::fromStdin() {
	return StringFactory::fromStream(std::cin);
}

String StringFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return StringFactory::fromString(input);
}

String StringFactory::parse(std::list<sax::Token> tokens) {
	StringParser parser;
	return parser.parse(tokens);
}

} /* namespace string */
} /* namespace notions */
