/*
 * AutomatonParser.h
 *
 *  Created on: Oct 12, 2013
 *      Author: martin
 */

#ifndef AUTOMATONPARSER_H_
#define AUTOMATONPARSER_H_

#include "UnknownAutomaton.h"

#include <list>
#include <set>
#include "../sax/Token.h"

namespace automaton {

using namespace std;
using namespace sax;

/**
 * Parser used to get UnknownAutomaton from XML parsed into list of Tokens.
 */
class AutomatonParser {
protected:
	static void parseStates(list<Token> &input, UnknownAutomaton& automaton);
	static void parseInputAlphabet(list<Token> &input, UnknownAutomaton& automaton);
	static void parseInitialStates(list<Token> &input, UnknownAutomaton& automaton);
	static void parseFinalStates(list<Token> &input, UnknownAutomaton& automaton);
	static void parseStackAlphabet(list<Token> &input, UnknownAutomaton& automaton);
	static void parseStartSymbols(list<Token> &input, UnknownAutomaton& automaton);
	static void parseTapeAlphabet(list<Token> &input, UnknownAutomaton& automaton);
	static void parseBlankSymbol(list<Token> &input, UnknownAutomaton& automaton);

	static void parseTransitions(list<Token> &input, UnknownAutomaton& automaton);
	static UnknownTransition parseTransition(list<Token>& input);
	static void parsePop(list<Token>& input, UnknownTransition* transition);
	static void parsePush(list<Token>& input, UnknownTransition* transition);

	static State parseState(list<Token> &input, string tagName = "state");
	static Symbol parseSymbol(list<Token> &input, string tagName = "symbol");
	static Shift parseShift(list<Token> &input);

	static bool isToken(list<Token> &input, Token::TokenType type, string data);
	static void popToken(list<Token> &input, Token::TokenType type, string data);

public:
	/**
	 * Parses the xml and returns the automaton. The input is destroyed in the process.
	 * @param input XML represented as list of Tokens
	 * @return UnknownAutomaton
	 * @throws ParserException when an error occurs
	 */
	static UnknownAutomaton parse(list<Token> &input);
};

} /* namespace automaton */
#endif /* AUTOMATONPARSER_H_ */
