/*
 * Concatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: martin
 */

#ifndef CONCATENATION_H_
#define CONCATENATION_H_

#include <list>
#include "RegExpElement.h"

namespace regexp {

using namespace std;

/**
 * Represents concatenation operator in the regular expression. Contains list of RegExpElement
 * as operands of the operator.
 */
class Concatenation: public RegExpElement {
private:
	list<RegExpElement*> elements;
public:
	Concatenation();
	Concatenation(const Concatenation& other);
	Concatenation& operator =(const Concatenation& other);
	~Concatenation();

	/**
	 * @return list of operands
	 */
	list<RegExpElement*>& getElements();

	/**
	 * @return list of operands
	 */
	const list<RegExpElement*>& getElements() const;

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	RegExpElement* clone() const;

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator==(const Concatenation&) const;

	/**
	 * @copydoc RegExpElement::getAlphabet()
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */
#endif /* CONCATENATION_H_ */
