/*
 * Iteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "Iteration.h"
#include <cstdio>

namespace regexp {

Iteration::Iteration() {
	element = NULL;
}

Iteration::Iteration(const Iteration& other) {
	if (other.element != NULL) {
		element = other.element->clone();
	} else {
		element = NULL;
	}

}

Iteration& Iteration::operator=(const Iteration& other) {
	if (this == &other) {
		return *this;
	}

	if (element != NULL) {
		delete element;
	}

	if (other.element != NULL) {
		element = other.element->clone();
	} else {
		element = NULL;
	}

	return *this;
}

regexp::Iteration::~Iteration() {
	if (element != NULL) {
		delete element;
	}

	element = NULL;
}

RegExpElement* Iteration::getElement() {
	return element;
}

const RegExpElement* Iteration::getElement() const {
	return element;
}

void Iteration::setElement(RegExpElement* element) {
	this->element = element;
}

RegExpElement* Iteration::clone() const {
	return new Iteration(*this);
}

bool Iteration::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool Iteration::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool Iteration::operator>(const RegExpElement& other) const {
	return other < *this;
}

bool Iteration::operator<(const Concatenation&) const {
	return true;
}

bool Iteration::operator<(const Alternation&) const {
	return true;
}

bool Iteration::operator<(const Iteration& other) const {
	return *(this->element) < *(other.element);
}

bool Iteration::operator==(const Iteration& other) const {
	return *(this->element) == *(other.element);
}

void Iteration::getAlphabet( std::set<alphabet::Symbol> & alphabet ) const {
	element->getAlphabet( alphabet );
}

bool Iteration::containsEmptyString() const {
	return true;
}

bool Iteration::isEmpty() const {
	return false;
}

} /* namespace regexp */

