    /*
 * RegExp.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXP_H_
#define REGEXP_H_

#include <vector>
#include <list>
#include <string>
#include "RegExpElement.h"
#include "RegExpEmpty.h"

#include "../alphabet/Symbol.h"

namespace regexp {

using namespace std;

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of RegExpElement.
 */
class RegExp {
private:
	RegExpElement* regExp;

public:
	RegExp();
	RegExp(const RegExpElement* regExp);

	/**
	 * Copy constructor.
	 * @param other RegExp to copy
	 */
	RegExp(const RegExp& other);
	RegExp& operator =(const RegExp& other);
	~RegExp();

	/**
	 * @return Root node of the regular expression tree
	 */
	RegExpElement* getRegExp();

	/**
	 * @return Root node of the regular expression tree
	 */
	const RegExpElement* getRegExp() const;

	/**
	 * Sets the root node of the regular expression tree. Doesn't perform copy of the regExp param,
	 * just stores it!
	 * @param regExp root node to set
	 */
	void setRegExp(RegExpElement* regExp);

	/**
	 * Gets alphabet symbols used in RegExp.
	 * @return set of alphabet symbols used in regexp.
	 */
	std::set<alphabet::Symbol> getAlphabet(void) const;

	/**
	 * @return true if regexp represents empty language
	 */
	bool isEmpty() const;

	/**
	 * @return true if regexp matches empty string (epsilon)
	 */
	bool containsEmptyString() const;

	/**
	 * Prints XML representation of the RegExp to the output stream.
	 * @param out output stream to which print the RegExp
	 */
	void toXML(ostream& out) const;

	/**
	 * Prints XML representation of the RegExp to the output stream.
	 * @param out output stream to which print the RegExp
	 * @param regexp RegExp to print
	 */
	friend ostream& operator<<(ostream& out, const RegExp& regexp);

	bool operator<(const RegExp&) const;
	bool operator<=(const RegExp&) const;
	bool operator==(const RegExp&) const;
	bool operator!=(const RegExp&) const;
	bool operator>(const RegExp&) const;
	bool operator>=(const RegExp&) const;
};

} /* namespace regexp */
#endif /* REGEXP_H_ */
