/*
 * RegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXPSYMBOL_H_
#define REGEXPSYMBOL_H_

#include <string>
#include "RegExpElement.h"
#include "../alphabet/Symbol.h"

namespace regexp {

using namespace std;
using namespace alphabet;

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class RegExpSymbol: public RegExpElement {
	string symbol;
public:
	RegExpSymbol();
	RegExpSymbol(const string& symbol);

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	RegExpElement* clone() const;

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator<(const RegExpSymbol&) const;
	virtual bool operator==(const RegExpSymbol&) const;

	/**
	 * @copydoc RegExpElement::getAlphabet()
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;

	/**
	 * @return Returns string representation of symbol.
	 */
	const string& getSymbol() const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */
#endif /* REGEXPSYMBOL_H_ */
