/*
 * String.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <algorithm>

#include "String.h"
#include "StringPrinter.h"
#include "../AlibException.h"

namespace notions {
namespace string {

String::String() {

}

void String::addAlphabetSymbol(const alphabet::Symbol& symbol) {
	std::pair<std::set<alphabet::Symbol>::iterator, bool> ret = this->alphabet.insert(symbol);
	if (!ret.second)
		throw alib::AlibException("Alphabet symbol \"" + symbol.getSymbol() + "\" already exists.");
}

void String::removeAlphabetSymbol(const alphabet::Symbol& symbol) {
	auto it = this->alphabet.find( symbol );
	if( it == this->alphabet.end())
		throw alib::AlibException("Symbol \"" + symbol.getSymbol() + "\" is not in alphabet.");

	auto it2 = std::find( this->symbols.begin( ), this->symbols.end( ), symbol );
	if(it2 != this->symbols.end())
		throw alib::AlibException("Symbol \"" + symbol.getSymbol() + "\" is used in string.");

	this->alphabet.erase(symbol);
}

const std::set<alphabet::Symbol>& String::getAlphabet() const {
	return this->alphabet;
}

std::list<alphabet::Symbol>& String::getContent() {
	return this->symbols;
}

const std::list<alphabet::Symbol>& String::getContent() const {
	return this->symbols;
}

bool String::isEmpty() const {
	return this->symbols.size() == 0;
}

void String::toXML(std::ostream& out) const {
	StringPrinter::toXML(*this, out);
}

std::ostream& operator <<(std::ostream& out, const String& string) {
	string.toXML(out);
	return out;
}

} /* namespace string */
} /* namespace notions */
