/*
 * String.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_H_
#define STRING_H_

#include <iostream>
#include <set>
#include <list>

#include "../alphabet/Symbol.h"

namespace notions {
namespace string {

/**
 * Represents regular expression parsed from the XML. Regular expression is stored
 * as a tree of StringElement.
 */
class String {
private:
	std::set<alphabet::Symbol> alphabet;
	std::list<alphabet::Symbol> symbols;

public:
	String();

	/**
	 * Adds input symbol to the alphabet.
	 * @param symbol Symbol to add
	 * @throws AlibException if symbol already exists
	 */
	void addAlphabetSymbol(const alphabet::Symbol& symbol);

	/**
	 * Removes input symbol from the the alphabet.
	 * @param symbol Symbol to remove
	 * @throws AlibException if symbol is not present in input alphabet
	 * or when symbol is present in content of the string.
	 */
	void removeAlphabetSymbol(const alphabet::Symbol& symbol);

	/**
	 * @return the input alphabet
	 */
	const std::set<alphabet::Symbol>& getAlphabet() const;

	/**
	 * @return List of symbols forming string.
	 */
	std::list<alphabet::Symbol>& getContent();

	/**
	 * @return List of symbols forming string (const version).
	 */
	const std::list<alphabet::Symbol>& getContent() const;

	/**
	 * @return true if string is an empty word (its length is 0)
	 */
	bool isEmpty() const;

	/**
	 * Prints XML representation of the String to the output stream.
	 * @param out output stream to which print the String
	 */
	void toXML(std::ostream& out) const;

	/**
	 * Prints XML representation of the String to the output stream.
	 * @param out output stream to which print the String
	 * @param string String to print
	 */
	friend std::ostream& operator<<(std::ostream& out, const String& string);

};

} /* namespace string */
} /* namespace notions */
#endif /* STRING_H_ */
