/*
 * AutomatonPrinter.h
 *
 *  Created on: Nov 11, 2013
 *      Author: martin
 */

#ifndef AUTOMATONPRINTER_H_
#define AUTOMATONPRINTER_H_

#include <ostream>
#include "UnknownAutomaton.h"
#include "FSM/ExtendedFSM.h"
#include "FSM/FSM.h"
#include "PDA/PDA.h"
#include "TM/TM.h"

namespace automaton {

using namespace std;

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class AutomatonPrinter {
protected:
	static const string INDENTATION;

	static void printStates(const set<State>& states, ostream& out, string tagName, string prefix);
	static void printAlphabet(const set<Symbol>& alphabet, ostream& out, string tagName, string prefix);

	static void printUnknownTransitions(const set<UnknownTransition>& transitions, ostream& out, string prefix);
	static void printTransitionsFSM(const std::set<automaton::TransitionFSM*, automaton::TransitionFSM::TransitionFSMPointerLess>& transitions, ostream& out, string prefix);
	static void printTransitionsPDA(const set<TransitionPDA>& transitions, ostream& out, string prefix);
	static void printTransitionsTM(const set<TransitionTM>& transitions, ostream& out, string prefix);


	static void printState(const State& state, ostream& out, string tagName, string prefix);
	static void printSymbol(const Symbol& symbol, ostream& out, string tagName, string prefix);
	static void printSymbolList(const list<Symbol>& symbols, ostream& out, string tagName, string prefix);
	static void printShift(const Shift& shift, ostream& out, string tagName, string prefix);

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @param out output stream to which print the automaton
	 */
	static void toXML(const UnknownAutomaton& automaton, ostream& out);

	/**
	 * Prints XML representation of FSM to the output stream.
	 * @param automaton automaton to print
	 * @param out output stream to which print the automaton
	 */
	static void toXML(const FSM& automaton, ostream& out);

	/**
	 * Prints XML representation of FSM to the output stream.
	 * @param automaton automaton to print
	 * @param out output stream to which print the automaton
	 */
	static void toXML(const ExtendedFSM& automaton, ostream& out);

	/**
	 * Prints XML representation of PDA to the output stream.
	 * @param automaton automaton to print
	 * @param out output stream to which print the automaton
	 */
	static void toXML(const PDA& automaton, ostream& out);

	/**
	 * Prints XML representation of TM to the output stream.
	 * @param automaton automaton to print
	 * @param out output stream to which print the automaton
	 */
	static void toXML(const TM& automaton, ostream& out);
};

} /* namespace automaton */
#endif /* AUTOMATONPRINTER_H_ */
