/*
 * FSM.cpp
 *
 *  Created on: Mar 25, 2013
 *      Author: martin
 */

#include "ExtendedFSM.h"
#include "../AutomatonPrinter.h"
#include "../exception/AutomatonException.h"
#include <ostream>

namespace automaton {

using namespace std;
using namespace alphabet;

ExtendedFSM::~ExtendedFSM() {

}

void ExtendedFSM::removeState(const State& state) {
	for (set<State>::const_iterator initialState = initialStates.begin(); initialState != initialStates.end();
			initialState++) {
		if (*initialState == state) {
			throw AutomatonException("State \"" + state.getName() + "\" is initial state.");
		}
	}

	for (set<State>::const_iterator finalState = finalStates.begin(); finalState != finalStates.end(); finalState++) {
		if (*finalState == state) {
			throw AutomatonException("State \"" + state.getName() + "\" is final state.");
		}
	}

	for (set<TransitionFSMExtended>::const_iterator t = transitions.begin(); t != transitions.end(); t++) {
		if (state == t->getFrom() || state == t->getTo())
			throw AutomatonException("State \"" + state.getName() + "\" is used in transition.");
	}

	int removed = states.erase(state);
	if (!removed)
		throw AutomatonException("State \"" + state.getName() + "\" doesn't exist.");
}

void ExtendedFSM::removeInputSymbol(const Symbol& symbol) {
	for (set<TransitionFSMExtended>::const_iterator transition = transitions.begin(); transition != transitions.end();
			transition++) {
		if (transition->isReading(symbol))
			throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" is used.");
	}

	int removed = inputAlphabet.erase(symbol);
	if (!removed)
		throw AutomatonException("Input symbol \"" + symbol.getSymbol() + "\" doesn't exist.");

}

void ExtendedFSM::addTransition(const State& from, const regexp::RegExp& input, const State& to) {
	addTransition(TransitionFSMExtended(from, input, to));
}

void ExtendedFSM::addTransition(const TransitionFSMExtended& transition) {
	const State& from = transition.getFrom();
	const regexp::RegExp& input = transition.getInput();
	const State& to = transition.getTo();

	if (states.find(from) == states.end())
		throw AutomatonException("State \"" + from.getName() + "\" doesn't exist.");

	/*if (inputAlphabet.find(input) == inputAlphabet.end())
		throw AutomatonException("Input symbol \"" + input.getSymbol() + "\" doesn't exist.");*/

	if (states.find(to) == states.end())
		throw AutomatonException("State \"" + to.getName() + "\" doesn't exist.");

	pair<set<TransitionFSMExtended>::iterator, bool> ret = transitions.insert(TransitionFSMExtended(transition));
	if (!ret.second)
		throw AutomatonException(
				"Transition (\"" + from.getName() + "\", \"Regexp\") -> \"" + to.getName()
						+ "\" already exists.");
}
void ExtendedFSM::removeTransition(const TransitionFSMExtended& transition) {
	/*int removed = transitions.erase(transition);
	if (!removed)
		throw AutomatonException(
				"Transition (\"" + transition.getFrom().getName() + "\", \"" + transition.getInput().getSymbol()
						+ "\") -> \"" + transition.getTo().getName() + "\" doesn't exist.");*/
}

const set<TransitionFSMExtended>& ExtendedFSM::getTransitions() const {
	return transitions;
}

const set<TransitionFSMExtended> ExtendedFSM::getTransitionsFromState(const State& from) const {
	if( states.find(from) == states.end())
		throw AutomatonException("State \"" + from.getName() + "\" doesn't exist");

	set<TransitionFSMExtended> transitionsFromState;
	for(auto const & transition : transitions){
		if (transition.getFrom() == from){
			transitionsFromState.insert(transition);
		}
	}

	return transitionsFromState;
}

const set<TransitionFSMExtended> ExtendedFSM::getTransitionsToState(const State& to) const {
	if( states.find(to) == states.end())
		throw AutomatonException("State \"" + to.getName() + "\" doesn't exist");

	set<TransitionFSMExtended> transitionsToState;
	for(auto const& transition : transitions){
		if (transition.getTo() == to){
			transitionsToState.insert(transition);
		}
	}

	return transitionsToState;
}

void ExtendedFSM::toXML(ostream& out) const {
	AutomatonPrinter::toXML(*this, out);
}

} /* namespace automaton */
