/*
 * EXTENDED_FSM.h
 *
 *  Created on: Mar 25, 2013
 *      Author: Jan Trávníček
 */

#ifndef EXTENDED_FSM_H_
#define EXTENDED_FSM_H_

#include <set>
#include "../Automaton.h"
#include "../State.h"
#include "../../alphabet/Symbol.h"
#include "../../regexp/RegExp.h"
#include "TransitionFSM.h"
#include "TransitionFSMExtended.h"
#include "TransitionFSMSymbol.h"
#include "TransitionFSMEpsilon.h"

namespace automaton {

using namespace std;
using namespace alphabet;

/**
 * Represents Finite State Machine.
 * Can store nondeterministic finite state machines with epsilon transitions.
 */
class ExtendedFSM: public Automaton {
protected:
	set<TransitionFSMExtended> transitions;
public:
	~ExtendedFSM();

	/**
	 * @copydoc Automaton::removeState(const State&)
	 */
	virtual void removeState(const State& state);

	/**
	 * @copydoc Automaton::removeInputSymbol(const Symbol&)
	 */
	virtual void removeInputSymbol(const Symbol& symbol);
	
	/**
	 * Adds transition defined by parameters to the automaton.
	 * @param current current state
	 * @param input input regexp
	 * @param next next state
	 * @throws AutomatonException when transition already exists or when transition contains state or symbol not present in the automaton
	 */
	void addTransition(const State& from, const regexp::RegExp& input, const State& to);

	/**
	 * Adds transition to the automaton.
	 * @param transition transition to add
	 * @throws AutomatonException when transition already exists or when transition contains state or symbol not present in the automaton
	 */
	void addTransition(const TransitionFSMExtended& transition);

	/**
	 * Removes transition from the automaton.
	 * @param transition transition to remove
	 * @throws AutomatonException when transition doesn't exists.
	 */
	void removeTransition(const TransitionFSMExtended& transition);

	/**
	 * @return automaton transitions
	 */
	const set<TransitionFSMExtended>& getTransitions() const;

	/**
	 * @return automaton transitions from state
	 */
	const set<TransitionFSMExtended> getTransitionsFromState(const State& from) const;

	/**
	 * @return automaton transitions to state
	 */
	const set<TransitionFSMExtended> getTransitionsToState(const State& to) const;

	/**
	 * @copydoc Automaton::toXML(ostream&) const
	 */
	virtual void toXML(ostream& out) const;
};

} /* namespace automaton */
#endif /* EXTENDED_FSM_H_ */
