/*
 * TransitionFSM.h
 *
 *  Created on: Mar 26, 2013
 *      Author: martin
 */

#ifndef TRANSITIONFSM_H_
#define TRANSITIONFSM_H_

#include "../Transition.h"
#include "../../alphabet/Symbol.h"
#include <ostream>

namespace automaton {

class TransitionFSMExtended;
class TransitionFSMSymbol;
class TransitionFSMEpsilon;

/**
 * Represents transition rule for FSM.
 */
class TransitionFSM : public Transition {
public:
  
	struct TransitionFSMPointerLess {
		bool operator() (const TransitionFSM* lhs, const TransitionFSM* rhs) const;
	};
	
	/**
	 * Creates new transition with given parameters.
	 * @param from from state
	 * @param to to state
	 */
	TransitionFSM(const State& from,const State& to);
	
	virtual bool isReading(const alphabet::Symbol& symbol) const = 0;

	// TransitionFSMEpsilon < TransitionFSMSymbol < TransitionFSMExtended
	virtual bool operator<(const TransitionFSM&) const = 0;
	virtual bool operator==(const TransitionFSM&) const = 0;
	virtual bool operator>(const TransitionFSM&) const = 0;

	virtual bool operator>=(const TransitionFSM&) const;
	virtual bool operator<=(const TransitionFSM&) const;
	virtual bool operator!=(const TransitionFSM&) const;
	
	virtual bool operator<(const TransitionFSMExtended&) const;
	virtual bool operator<(const TransitionFSMSymbol&) const;
	virtual bool operator<(const TransitionFSMEpsilon&) const;

	virtual bool operator==(const TransitionFSMExtended&) const;
	virtual bool operator==(const TransitionFSMSymbol&) const;
	virtual bool operator==(const TransitionFSMEpsilon&) const;
};

} /* namespace automaton */
#endif /* TRANSITIONFSM_H_ */
