/*
 * UnknownTransition.cpp
 *
 *  Created on: Oct 13, 2013
 *      Author: martin
 */

#include "UnknownTransition.h"

namespace automaton {

using namespace std;

UnknownTransition::UnknownTransition() :
		Transition(State(""), State("")), output(""), input("") {
	shift = NOT_SET;
}

const list<Symbol>& UnknownTransition::getPop() const {
	return pop;
}

void UnknownTransition::addPop(const Symbol& symbol) {
	pop.push_back(symbol);
}

const list<Symbol>& UnknownTransition::getPush() const {
	return push;
}

void UnknownTransition::addPush(const Symbol& symbol) {
	push.push_back(symbol);
}

const Symbol& UnknownTransition::getOutput() const {
	return output;
}

void UnknownTransition::setOutput(const Symbol& symbol) {
	output = symbol;
}

const Symbol& UnknownTransition::getInput() const {
	return output;
}

void UnknownTransition::setInput(const Symbol& symbol) {
	output = symbol;
}

const Shift& UnknownTransition::getShift() const {
	return shift;
}

void UnknownTransition::setShift(const Shift& shift) {
	this->shift = shift;
}

bool UnknownTransition::operator <(const UnknownTransition& other) const {
	if (from != other.from) {
		return from < other.from;
	} else if (input != other.input) {
		return input < other.input;
	} else if (to != other.to) {
		return to < other.to;
	} else if (output != other.output) {
		return output < other.output;
	} else if (shift != other.shift) {
		return shift < other.shift;
	}

	std::list<Symbol>::const_iterator it;
	std::list<Symbol>::const_iterator it2;

	//compare pop size
	if (pop.size() != other.pop.size())
		return pop.size() < other.pop.size();

	//compare pop content
	it = pop.begin();
	it2 = other.pop.begin();
	while (it != pop.end()) {
		if ((*it) != (*it2))
			return (*it) < (*it2);
		it++;
		it2++;
	}

	//compare push size
	if (push.size() != other.push.size())
		return push.size() < other.push.size();

	//compare push content
	it = push.begin();
	it2 = other.push.begin();
	while (it != push.end()) {
		if ((*it) != (*it2))
			return (*it) < (*it2);
		it++;
		it2++;
	}

	return false;
}

bool UnknownTransition::operator ==(const UnknownTransition& other) const {
	if (from == other.from && input == other.input && to == other.to && output == other.output
			&& shift == other.shift) {

		//compare pop size
		if (pop.size() != other.pop.size())
			return false;
		//compare push size
		if (push.size() != other.push.size())
			return false;

		std::list<Symbol>::const_iterator it;
		std::list<Symbol>::const_iterator it2;

		//compare pop content
		it = pop.begin();
		it2 = other.pop.begin();
		while (it != pop.end()) {
			if ((*it) != (*it2))
				return false;
			it++;
			it2++;
		}

		//compare push content
		it = push.begin();
		it2 = other.push.begin();
		while (it != push.end()) {
			if ((*it) != (*it2))
				return false;
			it++;
			it2++;
		}
		return true;
	} else {
		return false;
	}
}

bool UnknownTransition::operator !=(const UnknownTransition& other) const {
	return !((*this) == other);
}

std::ostream& UnknownTransition::operator>>(std::ostream& out) const {
	bool first;
	out << "UnknownTransition from = " << this->from
		<< " to = " << this->to
		<< " input = " << this->input
		<< " output = " << this->output
		<< " pop = [";

	first = true;
	for(list<Symbol>::const_iterator iter = this->pop.begin(); iter != this->pop.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}

	out << "] push = [";

	first = true;
	for(list<Symbol>::const_iterator iter = this->push.begin(); iter != this->push.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}
	out << "] shift = " << SHIFT_NAMES[this->shift];

	return out;
}

} /* namespace automaton */
