/*
 * UnknownTransition.h
 *
 *  Created on: Oct 13, 2013
 *      Author: Martin Zak
 */

#ifndef UNKNOWNTRANSITION_H_
#define UNKNOWNTRANSITION_H_

#include <list>

#include "Transition.h"
#include "State.h"
#include "../alphabet/Symbol.h"
#include "Shift.h"

namespace automaton {

using namespace std;
using namespace alphabet;

/**
 * Class representing unknown transition parsed from XML. Part of UnknownAutomaton.
 */
class UnknownTransition: public Transition {
protected:
	list<Symbol> pop;
	list<Symbol> push;

	Symbol output;
	Symbol input;
	Shift shift;

public:
	UnknownTransition();

	/**
	 * @return list of symbols that are popped from the stack
	 */
	const list<Symbol>& getPop() const;

	/**
	 * Add symbol to the end of the list of symbols that are popped from the stack.
	 * @param symbol Symbol to add
	 */
	void addPop(const Symbol& symbol);

	/**
	 * @return list of symbols that are pushed to the stack
	 */
	const list<Symbol>& getPush() const;

	/**
	 * Add symbol to the end of the list of symbols that are pushed to the stack.
	 * @param symbol Symbol to add
	 */
	void addPush(const Symbol& symbol);

	/**
	 * @return the output symbol of the transition
	 */
	const Symbol& getOutput() const;

	/**
	 * Sets the output Symbol of the transition.
	 * @param symbol Symbol to set
	 */
	void setOutput(const Symbol& symbol);

	/**
	 * @return the output symbol of the transition
	 */
	const Symbol& getInput() const;

	/**
	 * Sets the output Symbol of the transition.
	 * @param symbol Symbol to set
	 */
	void setInput(const Symbol& symbol);

	/**
	 * @return direction of movement of the reading head
	 */
	const Shift& getShift() const;

	/**
	 * Sets the direction of movement of reading head.
	 * @param shift the direction
	 */
	void setShift(const Shift& shift);

	bool operator <(const UnknownTransition& other) const;
	bool operator ==(const UnknownTransition& other) const;
	bool operator !=(const UnknownTransition& other) const;

	virtual std::ostream& operator>>(std::ostream&) const;
};

} /* namespace automaton */
#endif /* UNKNOWNTRANSITION_H_ */
