/*
 * Concatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef CONCATENATION_H_
#define CONCATENATION_H_

#include <vector>
#include <memory>
#include "RegExpElement.h"

namespace regexp {

/**
 * Represents concatenation operator in the regular expression. Contains list of RegExpElement
 * as operands of the operator.
 */
class Concatenation: public RegExpElement, public std::element<Concatenation, RegExpElement::visitor_type> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	std::vector<std::shared_ptr<RegExpElement> > elements;
public:
	Concatenation();
	Concatenation(const Concatenation& other);
	Concatenation(Concatenation&& other) noexcept;
	Concatenation& operator =(const Concatenation& other);
	Concatenation& operator =(Concatenation&& other) noexcept;
	virtual ~Concatenation() noexcept;
	
	/**
	 * @return elements
	 */
	const std::vector<std::shared_ptr<const RegExpElement> > & getElements() const;

	/**
	 * @param element to append
	 */
	void appendElement(const RegExpElement& element);
	
	void appendElement(RegExpElement&& element);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator==(const Concatenation&) const;

	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::getAlphabet() const
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* CONCATENATION_H_ */
