/*
 * Iteration.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "Iteration.h"
#include "../AlibException.h"

namespace regexp {

Iteration::Iteration() {
	element = std::shared_ptr<RegExpElement>(new RegExpEmpty());
}

Iteration::Iteration(const Iteration& other) : element(other.element->clone()) {

}

Iteration::Iteration(Iteration&& other) noexcept : element(other.element) {
	other.element = nullptr;
}

Iteration& Iteration::operator=(const Iteration& other) {
	if (this == &other) {
		return *this;
	}

	*this = Iteration(other);

	return *this;
}

Iteration& Iteration::operator=(Iteration&& other) noexcept {
	std::swap(this->element, other.element);

	return *this;
}

regexp::Iteration::~Iteration() noexcept {

}

const std::shared_ptr<const RegExpElement> & Iteration::getElement() const {
	return * reinterpret_cast<const std::shared_ptr<const RegExpElement> * >  (&element);
}

void Iteration::setElement(const RegExpElement& element) {
	this->element = std::shared_ptr<RegExpElement>(element.clone());
}

void Iteration::setElement(RegExpElement&& element) {
	this->element = std::shared_ptr<RegExpElement>(std::move(element).plunder());
}

RegExpElement* Iteration::clone() const {
	return new Iteration(*this);
}

RegExpElement* Iteration::plunder() && {
	return new Iteration(std::move(*this));
}

bool Iteration::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool Iteration::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool Iteration::operator>(const RegExpElement& other) const {
	return other < *this;
}

bool Iteration::operator<(const Concatenation&) const {
	return true;
}

bool Iteration::operator<(const Alternation&) const {
	return true;
}

bool Iteration::operator<(const Iteration& other) const {
	return *(this->element) < *(other.element);
}

bool Iteration::operator==(const Iteration& other) const {
	return *(this->element) == *(other.element);
}

void Iteration::operator>>(std::ostream& out) const {
	out << "(RegExpIteration " << *element << ")";
}

void Iteration::getAlphabet( std::set<alphabet::Symbol> & alphabet ) const {
	element->getAlphabet( alphabet );
}

bool Iteration::containsEmptyString() const {
	return true;
}

bool Iteration::isEmpty() const {
	return false;
}

} /* namespace regexp */

