/*
 * Iteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef ITERATION_H_
#define ITERATION_H_

#include <memory>
#include "RegExpElement.h"
#include "RegExpEmpty.h"

namespace regexp {

/**
 * Represents iteration operator in the regular expression. Contains one RegExpElement
 * as operand.
 */
class Iteration: public RegExpElement, public std::element<Iteration, RegExpElement::visitor_type> {
protected:
	std::shared_ptr<RegExpElement> element;

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;
public:
	Iteration();
	Iteration(const Iteration& other);
	Iteration(Iteration&& other) noexcept;
	Iteration& operator =(const Iteration& other);
	Iteration& operator =(Iteration&& other) noexcept;
	virtual ~Iteration() noexcept;

	/**
	 * @return element
	 */
	const std::shared_ptr<const RegExpElement> & getElement() const;
	
	/**
	 * @param element to iterate
	 */
	void setElement(const RegExpElement& element);
	
	void setElement(RegExpElement&& element);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator==(const Iteration&) const;

	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::getAlphabet() const
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* ITERATION_H_ */
