/*
 * RegExp.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExp.h"
#include "../AlibException.h"

#include <iostream>

namespace regexp {

RegExp::RegExp() {
	this->regExp = std::shared_ptr<RegExpElement>(new RegExpEmpty());
}

RegExp::RegExp(const RegExpElement& regExp) {
	this->regExp = std::shared_ptr<RegExpElement>(regExp.clone());
}

RegExp::RegExp(const RegExp& other) : regExp(other.regExp->clone()) {

}

RegExp::RegExp(RegExp&& other) noexcept : regExp(other.regExp) {
	other.regExp = NULL;
}

RegExp& RegExp::operator=(const RegExp& other) {
	if (this == &other) {
		return *this;
	}

	*this = RegExp(other);

	return *this;
}

RegExp& RegExp::operator=(RegExp&& other) noexcept {
	std::swap(this->regExp, other.regExp);

	return *this;
}

RegExp::~RegExp() noexcept {

}

const std::shared_ptr<const RegExpElement> RegExp::getRegExp() const {
	return std::shared_ptr<const RegExpElement>(regExp);
}

void RegExp::setRegExp(const RegExpElement& regExp) {
	this->regExp = std::shared_ptr<RegExpElement>(regExp.clone());
}

void RegExp::setRegExp(RegExpElement&& regExp) {
	this->regExp = std::shared_ptr<RegExpElement>(std::move(regExp).plunder());
}

std::set<alphabet::Symbol> RegExp::getAlphabet() const {
	std::set<alphabet::Symbol> alphabet;

	regExp->getAlphabet( alphabet );

	return alphabet;
}

bool RegExp::isEmpty() const {
	return regExp->isEmpty();
}

bool RegExp::containsEmptyString() const {
	return regExp->containsEmptyString();
}

std::ostream& operator <<(std::ostream& out, const RegExp& regExp) {
	out << "(RegExp " << *(regExp.regExp) << ")";
	return out;
}

bool RegExp::operator<(const RegExp& other) const {
	return *(this->regExp) < *(other.regExp);
}

bool RegExp::operator<=(const RegExp& other) const {
	return *(this->regExp) <= *(other.regExp);
}

bool RegExp::operator==(const RegExp& other) const {
	return *(this->regExp) == *(other.regExp);
}

bool RegExp::operator!=(const RegExp& other) const {
	return *(this->regExp) != *(other.regExp);
}

bool RegExp::operator>(const RegExp& other) const {
	return *(this->regExp) > *(other.regExp);
}

bool RegExp::operator>=(const RegExp& other) const {
	return *(this->regExp) >= *(other.regExp);
}

} /* namespace regexp */
