/*
 * RegExpEmpty.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_EMPTY_H_
#define REG_EXP_EMPTY_H_

#include "RegExpElement.h"

namespace regexp {

/**
 * Represents empty regular expression in the regular expression.
 */
class RegExpEmpty: public RegExpElement, public std::element<RegExpEmpty, RegExpElement::visitor_type> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

public:
	RegExpEmpty();

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator<(const RegExpSymbol&) const;
	virtual bool operator<(const RegExpEpsilon&) const;
	virtual bool operator==(const RegExpEmpty&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::getAlphabet() const
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* REG_EXP_EMPTY_H_ */
