/*
 * RegExpFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_FROM_XML_PARSER_H_
#define REG_EXP_FROM_XML_PARSER_H_

#include "RegExp.h"
#include "RegExpElements.h"
#include "../sax/Token.h"

namespace regexp {

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpFromXMLParser {
	RegExpElement* parseElement(std::list<sax::Token>& input);

	RegExpEpsilon* parseEpsilon(std::list<sax::Token>& input);
	RegExpEmpty* parseEmpty(std::list<sax::Token>& input);
	RegExpSymbol* parseSymbol(std::list<sax::Token> &input);
	Iteration* parseIteration(std::list<sax::Token> &input);
	Alternation* parseAlternation(std::list<sax::Token> &input);
	Concatenation* parseConcatenation(std::list<sax::Token> &input);
	bool isToken(std::list<sax::Token> &input, sax::Token::TokenType type, std::string data);
	void popToken(std::list<sax::Token> &input, sax::Token::TokenType type, std::string data);

public:
	/**
	 * Parses the XML and returns regular expression. The input is destroyed in the process.
	 * @param input XML represented as list of tokens
	 * @return RegExp
	 * @throws ParserException when an error occurs
	 */
	RegExp parse(std::list<sax::Token>& input);
};

} /* namespace regexp */
#endif /* REG_EXP_FROM_PARSER_H_ */
