/*
 * RegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpSymbol.h"

namespace regexp {

RegExpSymbol::RegExpSymbol(const std::string& symbol) :
		symbol(symbol) {
}

RegExpSymbol::RegExpSymbol(std::string&& symbol) :
		symbol(std::move(symbol)) {
}

RegExpElement* RegExpSymbol::clone() const {
	return new RegExpSymbol(*this);
}

RegExpElement* RegExpSymbol::plunder() && {
	return new RegExpSymbol(std::move(*this));
}

bool RegExpSymbol::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool RegExpSymbol::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool RegExpSymbol::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool RegExpSymbol::operator<(const Concatenation&) const {
	return true;
}

bool RegExpSymbol::operator<(const Alternation&) const {
	return true;
}

bool RegExpSymbol::operator<(const Iteration&) const {
	return true;
}

bool RegExpSymbol::operator<(const RegExpSymbol& other) const {
	return this->symbol < other.symbol;
}

bool RegExpSymbol::operator==(const RegExpSymbol& other) const {
	return this->symbol == other.symbol;
}

void RegExpSymbol::operator>>(std::ostream& out) const {
	out << "(RegExpSymbol " << symbol << ")";
}

bool RegExpSymbol::containsEmptyString() const {
	return false;
}

bool RegExpSymbol::isEmpty() const {
	return false;
}

void RegExpSymbol::getAlphabet( std::set<alphabet::Symbol> & alphabet ) const {
	alphabet.insert( alphabet::Symbol( this->getSymbol( ) ) );
}

const std::string& RegExpSymbol::getSymbol() const {
	return this->symbol;
}

} /* namespace regexp */

