/*
 * RegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_SYMBOL_H_
#define REG_EXP_SYMBOL_H_

#include <string>
#include "RegExpElement.h"
#include "../alphabet/Symbol.h"

namespace regexp {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class RegExpSymbol : public RegExpElement, public std::element<RegExpSymbol, RegExpElement::visitor_type> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	std::string symbol;
public:
	RegExpSymbol(const std::string& symbol);
	RegExpSymbol(std::string&& symbol);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator<(const Iteration&) const;
	virtual bool operator<(const RegExpSymbol&) const;
	virtual bool operator==(const RegExpSymbol&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::getAlphabet() const
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * Returns the string representation of RegExp Symbol.
	 */
	const std::string& getSymbol() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* REG_EXP_SYMBOL_H_ */
