/*
 * RegExpToXMLPrinter.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpToXMLPrinter.h"

namespace regexp {

RegExpToXMLPrinter::RegExpToXMLPrinter(std::ostream& out) : m_Out(out) {

}

void RegExpToXMLPrinter::Visit(const RegExp::element_type& regexp) {
	m_Out << "<regexp>" << std::endl;
	regexp.Accept(*this);
	m_Out << "</regexp>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const RegExpElement::element_type& element) {
	element.Accept(*this);
}

void RegExpToXMLPrinter::Visit(const Alternation& alternation) {
	m_Out << "<alternation>" << std::endl;
	for (const auto& element : alternation.getElements()) {
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		object.Accept(*this);
	}
	m_Out << "</alternation>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const Concatenation& concatenation) {
	m_Out <<"<concatenation>" << std::endl;
	for (auto element : concatenation.getElements()) {
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		object.Accept(*this);
	}
	m_Out <<"</concatenation>" << std::endl;

}

void RegExpToXMLPrinter::Visit(const Iteration& iteration) {
	m_Out << "<iteration>" << std::endl;
	const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*iteration.getElement());
	object.Accept(*this);
	m_Out << "</iteration>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const RegExpSymbol& symbol) {
	m_Out << "<symbol>";
	m_Out << symbol.getSymbol();
	m_Out << "</symbol>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const RegExpEpsilon& epsilon) {
	m_Out << "<epsilon>";
	m_Out << "</epsilon>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const RegExpEmpty& empty) {
	m_Out << "<empty>";
	m_Out << "</empty>" << std::endl;
}

void RegExpToXMLPrinter::Visit(const RegExp& regexp) {
	m_Out << "<regexp>" << std::endl;
	regexp.getRegExp()->Accept(*this);
	m_Out << "</regexp>" << std::endl;
}

} /* namespace regexp */
