/*
 * RegExpToXMLPrinter.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REGEXP_TO_XML_PRINTER_H_
#define REGEXP_TO_XML_PRINTER_H_

#include <ostream>
#include "RegExp.h"
#include "RegExpElements.h"
#include "../std/visitor.hpp"

namespace regexp {

/**
 * This class contains methods to print XML representation of regular expression to the output stream.
 */
class RegExpToXMLPrinter : public RegExp::visitor_type, public RegExpElement::visitor_type {

	std::ostream& m_Out;

public:
	RegExpToXMLPrinter(std::ostream& out);

	void Visit(const Alternation& alternation);
	void Visit(const Concatenation& concatenation);
	void Visit(const Iteration& iteration);
	void Visit(const RegExpSymbol& symbol);
	void Visit(const RegExpEpsilon& epsilon);
	void Visit(const RegExpEmpty& empty);
	
	void Visit(const RegExp& regexp);
	
	/**
	 * Prints XML representation of RegExp to the output stream.
	 * @param regexp RegExp to print
	 * @param out output stream to which print the RegExp
	 */
	void Visit(const RegExp::element_type& regexp);
	
	void Visit(const RegExpElement::element_type& element);
};

} /* namespace regexp */

#endif /* REGEXP_TO_XML_PRINTER_H_ */
