/*
 * SaxInterface.h
 *
 *  Created on: 8.8.2012
 *      Author: Martin Zak
 */

#ifndef SAX_INTERFACE_H_
#define SAX_INTERFACE_H_

#include <libxml/parser.h>
#include <list>
#include "Token.h"

namespace sax {

/**
 * This class performs parsing of file or string containing XML. Contains callback
 * methods for libxml SAX parser.
 */
class SaxInterface {

protected:
	/**
	 * Initializes the SAX parser.
	 */
	static void initSAXHandler(xmlSAXHandler&);

	/**
	 * Callback method called when charactes (between tags) are read.
	 * @param userData contains list of parsed tokens
	 * @param ch array of parsed characters
	 * @param len length of the array
	 */
	static void characters(void * userData, const xmlChar * ch, int len);

	/**
	 * Callback method called when start of the tag is read.
	 * @param userData contains list of parsed tokens
	 * @param name array of characters containing name of the tag
	 * @param attrs array containing attributes (arrays of characters) of the tag
	 */
	static void startElement(void *userData, const xmlChar *name, const xmlChar **attrs);

	/**
	 * Callback method called when end of the tag is read.
	 * @param userData contains list of parsed tokens
	 * @param name array of characters containing name of the tag
	 */
	static void endElement(void * userData, const xmlChar * name);

public:
	/**
	 * Parses the string containing XML.
	 * @param xmlIn input XML
	 * @param out list of tokens that are returned
	 * @throws AlibException when an error occurs (e.g. XML is not valid)
	 */
	static void parseMemory(const std::string& xmlIn, std::list<Token>& out);

	/**
	 * Parses the file containing XML.
	 * @param filename input XML
	 * @param out list of tokens that are returned
	 * @throws AlibException when an error occurs (e.g. file doesn't exist, XML is not valid)
	 */
	static void parseFile(const std::string& filename, std::list<Token>& out);
};

} /* namespace sax */

#endif /* SAX_INTERFACE_H_ */
