/*
 * Visitor.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Andrew Durward
 *      Modified: Jan Travnicek
 */

#ifndef VISITOR_H_
#define VISITOR_H_

#include <tuple>

namespace std {

// Visitor template declaration
template<typename... Types>
class visitor;

// specialization for single type    
template<typename T>
class visitor<T> {
public:
    virtual void Visit(const T & visitable) = 0;
};

// specialization for multiple types
template<typename T, typename... Types>
class visitor<T, Types...> : public visitor<Types...> {
public:
    // promote the function(s) from the base class
    using visitor<Types...>::Visit;

    virtual void Visit(const T & visitable) = 0;
};

template<typename VisitorType>
class elementBase {
public:
    typedef VisitorType visitor_type;
    typedef elementBase element_type;

    virtual void Accept(VisitorType& visitor) const = 0;
};

template<typename Derived, typename VisitorType>
class element : virtual public elementBase<VisitorType> {
public:
    virtual void Accept(VisitorType& visitor) const {
        visitor.Visit(static_cast<const Derived&>(*this));
    }
};

} /* namespace std */

#endif /* VISITOR_H_ */