/*
 * TrimFSM.cpp
 *
 *  Created on: 23. 3. 2014
 *      Author: tomas
 */

#include "TrimFSM.h"

using namespace alib;
using namespace automaton;
using namespace std;

FSM TrimFSM::removeDeadStates( const FSM & fsm )
{
    // 1a
    deque<set<State>> Qi;
    Qi.push_back( set<State>( ) );
    Qi.at( 0 ) = fsm.getFinalStates( );

    int i = 1;

    // 1bc
    while( true )
    {
        Qi.push_back( Qi.at( i - 1 ) ); // copy Qi-1 into Qi
        for( const auto & p : Qi.at( i - 1 ) )
            for( const auto & t : fsm.getTransitionsToState( p ) )
                Qi.at( i ).insert( t.getFrom( ) );

        if( Qi.at( i ) == Qi.at( i - 1 ) )
            break;

        i = i + 1;
    }
    const set<State> Qu = Qi.at( i );

    // 2.

    FSM M;

    for( const auto & q : Qu )
        M.addState( q );

    for( const auto & a : fsm.getInputAlphabet( ) )
        M.addInputSymbol( a );

    for( const auto & t : fsm.getTransitions( ) )
    {
        if( isInSet( t.getFrom( ), Qu ) && isInSet( t.getTo( ), Qu ) )
        {
            M.addTransition( t );
        }
    }

    for( const auto & q : fsm.getInitialStates( ) )
        M.addInitialState( q );

    for( const auto & q : fsm.getFinalStates( ) )
        M.addFinalState( q );

    return M;
}


FSM TrimFSM::removeUnreachableStates( const FSM & fsm )
{
    if( fsm.getInitialStates( ).size( ) != 1 )
        throw AlibException( "NFA must have exactly one initial state." );

    // 1a
    deque<set<State>> Qi;
    Qi.push_back( set<State>( ) );
    Qi.at( 0 ).insert( * fsm.getInitialStates( ).begin( ) );

    int i = 1;

    // 1bc
    while( true )
    {
        Qi.push_back( Qi.at( i - 1 ) );

        for( const auto & p : Qi.at( i - 1 ) )
        {
            for( const auto & transition : fsm.getTransitionsFromState( p ) )
            {
                Qi.at( i ).insert( transition.getTo( ) );
            }
        }

        if( Qi.at( i ) == Qi.at( i - 1 ) )
            break;

        i = i + 1;
    }

    const set<State> Qa = Qi.at( i );


    // 2

    FSM M;

    for( const auto & q : Qa )
        M.addState( q );
    for( const auto & a : fsm.getInputAlphabet( ) )
        M.addInputSymbol( a );

    for( const auto & transition : fsm.getTransitions( ) )
        if( isInSet( transition.getFrom( ), Qa ) )
            M.addTransition( transition );

    M.addInitialState( * fsm.getInitialStates( ).begin( ) );

    set<State> intersect;
    set_intersection( fsm.getFinalStates( ).begin(), fsm.getFinalStates( ).end(), Qa.begin( ), Qa.end( ), inserter( intersect, intersect.begin( ) ) );
    for( auto const & state : intersect )
        M.addFinalState( state );

    return M;
}
