/*
 * ContextFreeGrammarTransformations.cpp
 *
 *  Created on: 22. 3. 2014
 *      Author: tomas
 */

#include "ContextFreeGrammarTransformations.h"

using namespace alib;
using namespace grammar;
using namespace std;

set<Symbol> ContextFreeGrammarTransformations::getProductiveNonTerminals( const ContextFreeGrammar & grammar )
{
    // 1.
    deque<set<Symbol>> Ni;
    Ni.push_back( set<Symbol>( ) );

    int i = 1;

    // 2.
    while( true )
    {
        Ni.push_back( Ni.at( i - 1 ) );
        bool valid = true;

        for( const auto & rule : grammar.getRules( ) )
        {
            for( const auto & symbol : rule.getRightSide( ) )
            {
                if( ! isInSet( symbol, Ni.at( i - 1 ) ) && ! isInSet( symbol, grammar.getTerminalSymbols( ) ) )
                {
                    valid = false;
                    break;
                }
            }

            if( valid )
                Ni.at( i ).insert( rule.getLeftSide( ).front( ) );
        }

        if( Ni.at( i ) == Ni.at( i - 1 ) )
            break;

        i = i + 1;
    }

    // 3.
    return Ni.at( i );
}

bool ContextFreeGrammarTransformations::isLanguageEmpty( const grammar::ContextFreeGrammar & grammar )
{
    return isInSet( grammar.getStartSymbol( ), getProductiveNonTerminals( grammar ) );
}

ContextFreeGrammar ContextFreeGrammarTransformations::removeUnreachableSymbols( const ContextFreeGrammar & grammar )
{
    // 1.
    deque<set<Symbol>> Vi;
    Vi.push_back( set<Symbol>( ) );
    Vi.at( 0 ).insert( grammar.getStartSymbol( ) );

    int i = 1;

    // 2.
    while( true )
    {
        Vi.push_back( Vi.at( i - 1 ) );

        for( const auto & rule : grammar.getRules( ) )
        {
            if( isInSet( rule.getLeftSide( ).front( ), Vi.at( i - 1 ) ) )
            {
                for( const auto & symbol : rule.getRightSide( ) )
                {
                    Vi.at( i ).insert( symbol );
                }
            }
        }


        if( Vi.at( i ) == Vi.at( i - 1 ) )
            break;

        i = i + 1;
    }

    // 3.
    ContextFreeGrammar ret;

    set<Symbol> newNonTerminals, newTerminals;

    set_intersection( Vi.at( i ).begin( ), Vi.at( i ).end( ), grammar.getNonTerminalSymbols( ).begin( ), grammar.getNonTerminalSymbols( ).end( ), std::inserter( newNonTerminals, newNonTerminals.begin( ) ) );
    for( const auto & symbol : newNonTerminals )
        ret.addNonTerminalSymbol( symbol );

    set_intersection( Vi.at( i ).begin( ), Vi.at( i ).end( ), grammar.getTerminalSymbols( ).begin( ), grammar.getTerminalSymbols( ).end( ), std::inserter( newTerminals, newTerminals.begin( ) ) );
    for( const auto & symbol : newTerminals )
        ret.addTerminalSymbol( symbol );

    // A->\alpha: if A \in N' and \alpha in V_i*, then A->\alpha in P
    for( const auto & rule : grammar.getRules( ) )
    {
        if( isInSet( rule.getLeftSide( ).front( ) , ret.getNonTerminalSymbols( ) ) &&
            all_of( rule.getRightSide( ).begin( ), rule.getRightSide( ).end( ), [ Vi, i ]( const Symbol & symb ){ return isInSet( symb, Vi.at( i ) ); } ) )
        {
            ret.addRule( rule );
        }
    }

    ret.setStartSymbol( grammar.getStartSymbol( ) );

    return ret;
}

ContextFreeGrammar ContextFreeGrammarTransformations::removeUnproductiveSymbols( const ContextFreeGrammar & grammar )
{
    // 1.
    set<Symbol> Nt = getProductiveNonTerminals( grammar );
    ContextFreeGrammar G1;

    for( const auto & symbol : Nt )
        G1.addNonTerminalSymbol( symbol );

    for( const auto & symbol : grammar.getTerminalSymbols( ) )
        G1.addTerminalSymbol( symbol );

    const set<Symbol> & T = G1.getTerminalSymbols( );
    for( const auto & rule : grammar.getRules( ) )
    {
        if( isInSet( rule.getLeftSide( ).front( ), Nt ) &&
            all_of( rule.getRightSide( ).begin( ), rule.getRightSide( ).end( ), [ Nt, T ]( const Symbol & symbol ){ return isInSet( symbol, Nt ) || isInSet( symbol, T ); } ) )
        {
            G1.addRule( rule );
        }
    }

    if( ! isInSet( grammar.getStartSymbol( ), G1.getNonTerminalSymbols( ) ) )
        throw AlibException( "Starting symbol of grammar was marked as unproductive and therefore it was removed." );

    G1.setStartSymbol( grammar.getStartSymbol( ) );

    // 2.
    return removeUnreachableSymbols( G1 );
}
